#!/usr/bin/env python3.2
# -*- coding: utf-8 -*-
#
#       bezier_cubique.py
#
# ************** importation ***************
from tkinter import *
from tkinter.font import Font
from threading import Thread, Lock
import sys
# ************* fin de importation *********
print (sys.version)
# ********************************************************
# ************ une classe pour les ressources ************
# ********************************************************
class ClasseRessources () :
    def __init__ (self, hauteurEcran) :
        # dimensions
        self.hauteur = hauteurEcran * 5 // 6
        self.bord = self.hauteur // 28
        self.carre = self.hauteur - 2*self.bord
        self.couleurPanneau = "#ffe4b5"
        self.couleurCanevas = "#e0e0ff"
        self.fonteTitre = Font (family="Helvetica", 
                        size = self.hauteur // 25, weight = "bold")
        self.fonteUsuel = Font (family="Helvetica", 
                              size = self.hauteur // 40)
        self.fonteScale = Font (family="Helvetica", 
                                          size = self.hauteur // 60)
        self.fonteGraduation = Font (family="Helvetica", 
                                   size = self.hauteur // 70)
        self.verrou = Lock()
        self.bgStopGoDefaut = None # couleur par défaut initialisé dans Application
        self.bgStopGoTrace = "#a0ffa0"  # vert
        self.bgStopGoFinTrace = "#a0ffff" # cyan
        self.bgStopGoArret = "#ffa0a0"    # rouge
        self.message = ["Point origine","Point de contôle 1","Point de contôle 2",
               "Point terminal"]
        self.presentation ="""Une courbe de Bézier cubique est définie par 4 points.
L'origine et le point terminal de la courbe sont repérés des
par des carrés rouges et les tangentes en ces points par
traits verts se terminant aux deux autres points (rond bleu)\n
Pour commencer le tracé, cliquer sur ""(re)INITIALISER"" et
amener la souris sur le carré de tracé avec des axes gradués.\n\n
Les graduations et les coordonnées sont affichée avec une
échelle allant de 0 à 100.\n
Entrer (clic de la souris) les quatre points.\n
Le tracé peut être commandé par le bouton StopGo : tracé,
arrêt, recommencer. Les paramètres de 'pas' et de 'durée'
peuvent être réglés quand le tracé n'est plus en cours.\n
Le tracé par points est réalisé en figurant le point courant
comme barycentre des 4 points de définition ; les barycentres
partiels (par couples de points) sont figurés comme extrémités
des traits légers rouges.
"""
    def coord2str (CR, x, y) : # transforme les coordonnées canevas en chaîne
        x = (x - CR.bord) * 10000 // CR.carre
        y = (CR.carre + CR.bord-y-1)* 10000 // CR.carre
        return (str(x / 100), str(y / 100))

# ********************************************************
# ************ une classe pour les paramètres ************
# ********************************************************
class ClasseParametres ():
    def __init__ (self) :
    # il y a sept états possibles : -2 initial // -1 bouton initialiser 
    # 0 saisie de P0 // 1 saisie de P1 //2 saisie de P2 // 3 saisie de P3 
    # // 4 la courbe en train d'être tracée  // 5 fin de tracé
        self.etat = -2

    # les points : début (P0), contrôle (P1 & P2), terminal (P3)
        self.P0 = [0,0] # point : coordonnées canevas, calcul, affichage
        self.P1 = [0,0]
        self.P2 = [0,0]
        self.P3 = [0,0]
        self.Point = [self.P0, self. P1, self.P2, self.P3]

    # barres d'initialisation
        self.vitesse = 40
        self.pas = 50 
        self.masse = 25000
    # caractéristiques de courbe
        self.flagStopGo = False
        self.t = 0 # numéro du pas dans la liste suivante        
        self.Courbe = () # points de la courbe

    def setVitesse(self, v):
        self.vitesse = v * 10 # ajustement aux unités

    def setPas(self, p):
        self.masse = p*p*p
        self.pas = p # nombre de pas
        

# ********************************************************
# *************** les alias et constantes ****************
# ********************************************************
# CR = application.ressources  # alias pour simplifier l'écriture
# CP = appplication.paramètres # alias pour simplifier l'écriture
XC = 0 # coordonnée x sur le canevas
YC = 1 # coordonnée y sur le canevas

# ********************************************************
# *************** le canevas du dessin  ******************
# ********************************************************

class ClasseCanevas (Canvas) :
    def __init__ (self, panneau, application) :
        Canvas.__init__(self, panneau) # self : le canevas
        self.app = application
        CR = self.app.ressources # alias
        self.axes()
        self.bind("<Motion>", self.survol)
        self.bind("<Button-1>", self.pointsDeControle)
        self.create_text(2*CR.bord, 2*CR.bord, font=CR.fonteUsuel,
                   text=CR.presentation, tag="présentation", anchor=NW)
        self.create_text(CR.carre//2, 10*CR.bord, font=CR.fonteTitre,
                   text="ici", tag="présentation", anchor=NW,
                   fill = "#800000", activefill ="#ff0000")

    def axes (self) : # réalisation des axes et des graduations
        CR = self.app.ressources # alias
        r=CR.carre
        h=CR.hauteur
        b=CR.bord
        self.aire = self.create_rectangle (b,r+b,r+b,b,
                                  fill=CR.couleurPanneau,tags=("axes","carré") )
        self.create_line(0,r+b,h,r+b,width=3, arrow=LAST, tags="axes")
        self.create_line(b,h,b,0,width=3,arrow=LAST, tags="axes")
        self.create_text(b//2,h-b//2, font=CR.fonteUsuel, text="\u03a9")
        self.create_text(b//2,b//2, font=CR.fonteUsuel, text="y")
        self.create_text(h-b//2,h-b//2, font=CR.fonteUsuel, text="x")

        self.create_line (b+r//4,h-b//4,b+(r)//4,r+b,tags="axes")
        self.create_text(7*b, h-b//2, font=CR.fonteGraduation, text="25")
        self.create_line (b//4, b+3*r//4, b, b+3*r//4,tags="axes")
        self.create_text(7*b//8, 3*b//2+3*r//4, font=CR.fonteGraduation,
                          text="25",anchor=E)

        self.create_line (b+r//2,h-b//4,b+r//2,r+b,tags="axes")
        self.create_text(7*b+r//4, h-b//2, font=CR.fonteGraduation, 
                         text="50")
        self.create_line (b//4,b+r//2, b , b+r//2,tags="axes")
        self.create_text(7*b//8, 3*b//2+r//2, font=CR.fonteGraduation,
                          text="50",anchor=E)
        self.create_line (b+ 3*r//4,h-b//4,b+ 3*r//4,h-b,tags="axes")
        self.create_text(7*b+r//2, h-b//2, font=CR.fonteGraduation, 
                         text="75")
        self.create_line (b//4,b+r//4, b , b+r//4,tags="axes")
        self.create_text(7*b//8, 3*b//2+r//4, font=CR.fonteGraduation,
                         text="75",anchor=E)

        self.create_line (b+r,h-b//4,b+r,h-b,tags="axes")
        self.create_text(r, h-b//2, font=CR.fonteGraduation, text="100")
        self.create_line (b//4,b, b, b, tags="axes")
        self.create_text(7*b//8, 3*b//2, font=CR.fonteGraduation,
                           text="100",anchor=E)

    def pointsDeControle (self, event) :
        CR = self.app.ressources # alias
        CP = self.app.parametres # alias
        if (CP.etat<0 or CP.etat>3):
            return (None)        
        elif (CP.etat==0):
            self.create_rectangle (CP.P0[XC]-3,CP.P0[YC]-3,CP.P0[XC]+
            3,CP.P0[YC]+3,fill="", outline="red",width=3, tags="courbe")
            self.app.controles.setLabelPnormal (1)
        elif (CP.etat==1):
            self.create_line(CP.P0[XC],CP.P0[YC],CP.P1[XC],CP.P1[YC],
                                fill="#008000",width=3, tags="courbe")
            self.create_rectangle (CP.P0[XC]-3,CP.P0[YC]-3,CP.P0[XC]+3,
             CP.P0[YC]+3, fill="", outline="red",width=3, tags="courbe")
            self.create_oval (CP.P1[XC]-3,CP.P1[YC]-3,CP.P1[XC]+3,
              CP.P1[YC]+3, fill="",outline="blue",width=3, tags="courbe")
            self.app.controles.setLabelPnormal (2)
        elif (CP.etat==2):
            self.create_oval (CP.P2[XC]-3,CP.P2[YC]-3,CP.P2[XC]+3,
               CP.P2[YC]+3, fill="",outline="blue",width=3, tags="courbe")
            self.create_line(CP.P1[XC],CP.P1[YC],CP.P2[XC],CP.P2[YC],
                                fill="#808080",width=1, tags="courbe")
            self.app.controles.setLabelPnormal (3)
        elif (CP.etat==3):
            self.create_line(CP.P2[XC],CP.P2[YC],CP.P3[XC],CP.P3[YC],
                                  fill="#008000",width=3, tags="courbe")
            self.create_line(CP.P0[XC],CP.P0[YC],CP.P3[XC],CP.P3[YC],
                                  fill="#808080",width=1, tags="courbe")                                    
            self.create_line(CP.P1[XC],CP.P1[YC],CP.P3[XC],CP.P3[YC],
                                  fill="#808080",width=1, tags="courbe")
            self.create_line(CP.P0[XC],CP.P0[YC],CP.P2[XC],CP.P2[YC],
                                  fill="#808080",width=1, tags="courbe")                                                          
            self.create_oval (CP.P2[XC]-3,CP.P2[YC]-3,CP.P2[XC]+3,
               CP.P2[YC]+3,fill="", outline="blue",width=3, tags="courbe")
            self.create_rectangle (CP.P3[XC]-3,CP.P3[YC]-3,CP.P3[XC]+3,
                CP.P3[YC]+3,fill="", outline="red",width=3, tags="courbe")
            self.create_rectangle (CP.P0[XC]-3,CP.P0[YC]-3,CP.P0[XC]+3,
               CP.P0[YC]+3, fill="", outline="red",width=3, tags="courbe")
            self.create_oval (CP.P1[XC]-3,CP.P1[YC]-3,CP.P1[XC]+3,CP.P1[YC]+3, 
                         fill="" ,outline="blue",width=3, tags="courbe")         
            self.preparation()
        CP.etat = CP.etat+1

    def survol(self,event, listeRappel = [None, None]) :
        CR = self.app.ressources # alias
        CP = self.app.parametres # alias
        if (CP.etat==-2 or CP.etat>3):
            self.config(cursor="arrow")
            self.delete ("rappel")
            return (None)
        h = CR.hauteur
        b = CR.bord
        x = event.x
        y = event.y
        # se mettre dans le carré de référence
        if (x<b or x>h-b or y< b-1 or y>=h-b):
            self.config(cursor="arrow")
            if (listeRappel [0]):
                self.coords(listeRappel[XC],0,0,0,0)
                self.coords(listeRappel[YC],0,0,0,0)
            return (None)

        # initialiser
        if (CP.etat==-1):
            self.tag_lower ("présentation" , "carré")
            CP.etat=0 # initialiser
            listeRappel[XC] = self.create_line (CR.bord, y, x, y,
                                                tags="rappel")
            listeRappel[YC] = self.create_line (x, y, x,CR.carre+CR.bord,
                                                tags="rappel")
            self.app.controles.setLabelPnormal (0)
        self.config(cursor="crosshair")
        self.coords(listeRappel[XC],CR.bord, y, x, y)
        self.coords(listeRappel[YC],x, y, x,CR.carre+CR.bord)

        CP.Point[CP.etat][XC] = x
        CP.Point[CP.etat][YC] = y
        self.app.controles.setLabelPxy (CP.etat)

    def preparation (self):
        CP = self.app.parametres # alias
        self.app.controles.boutonStopGo.config(state=NORMAL)
        CP.t = 0        
        CP.Courbe = (CP.P0[XC],CP.P0[YC]) # point initial de la courbe

    def stopGo (self, listeCourbe =[None, None, None, None]) : 
        CR = self.app.ressources # alias 
        CP = self.app.parametres # alias      
        if (CP.etat==5):
           CP.etat=4 # état de tracé
           self.delete("lacourbe")
           self.preparation()

        while True :
          CR.verrou.acquire()
          fl = CP.flagStopGo
          CR.verrou.release()
          if (not fl):
              break

          CP.t = CP.t+1
          if (CP.t > CP.pas):
              break
          # pas suivant
          if (CP.t == CP.pas) : # point terminal
              CP.Courbe = CP.Courbe + (CP.P3[XC],CP.P3[YC])
              self.delete("lacourbe")
              listeCourbe[3] = self.create_line(CP.Courbe,fill="blue",
                       smooth=True, tags=("lacourbe","courbe"), width=2)
              self.app.controles.setLabelActuel(CP.P3[XC],CP.P3[YC])
              self.delete ("gdbhxx")
              CR.verrou.acquire()
              CP.flagStopGo=False
              CR.verrou.release()
              self.app.controles.normalDisabled(fin=False)
              self.app.controles.setBgBoutonStopGo (CR.bgStopGoFinTrace)
              CP.etat=5
              break
          p = CP.t
          q = CP.pas - CP.t          
          m0 = q*q*q
          m1 = 3*q*q*p
          m2 = 3*q*p*p
          m3 = p*p*p

          # les barycentres partiels : valeur m*x, m*y
          b = [ m0*CP.P0[XC], m0*CP.P0[YC], m1*CP.P1[XC], m1*CP.P1[YC],
                m2*CP.P2[XC], m2*CP.P2[YC], m3*CP.P3[XC], m3*CP.P3[YC] ]
          # coordonnées pour les lignes de barycentre partiel
          tuple_gd = ((b[0]+b[2])//(m0+m1),(b[1]+b[3])//(m0+m1),
                      (b[4]+b[6])//(m2+m3),(b[5]+b[7])//(m2+m3))
          tuple_bh = ((b[0]+b[6])//(m0+m3),(b[1]+b[7])//(m0+m3),
                      (b[2]+b[4])//(m1+m2),(b[3]+b[5])//(m1+m2))
          tuple_xx = ((b[0]+b[4])//(m0+m2),(b[1]+b[5])//(m0+m2),
                      (b[2]+b[6])//(m1+m3),(b[3]+b[7])//(m1+m3))            
          if (CP.t==1):
              listeCourbe[0]=self.create_line (
                          tuple_gd, fill="red", tags=("courbe", "gdbhxx"))
              listeCourbe[1]=self.create_line (
                          tuple_bh, fill="red", tags=("courbe", "gdbhxx"))              
              listeCourbe[2]=self.create_line (
                          tuple_xx, fill="red", tags=("courbe", "gdbhxx"))            
          else :
              self.coords(listeCourbe[0], tuple_gd)
              self.coords(listeCourbe[1], tuple_bh)
              self.coords(listeCourbe[2], tuple_xx)
              
          # un point de plus sur la courbe
          xc = (b[0]+b[2]+b[4]+b[6]) // CP.masse
          yc= (b[1]+b[3]+b[5]+b[7]) // CP.masse
          CP.Courbe = CP.Courbe + (xc,yc)
          self.delete("lacourbe")
          listeCourbe[3] = self.create_line(CP.Courbe, fill="blue",
                       smooth=True, tags=("lacourbe","courbe"), width=2)

          self.app.controles.setLabelActuel (xc, yc)
          self.update_idletasks() # forcer l'affichage
          self.after(CP.vitesse) # time.sleep() mais en millisecondes

# ********************************************************
# ********** les contrôles (boutons, messages) ***********
# ********************************************************
class ClasseControles () :
    def __init__(self,panneau, application) :
       self.app = application
       CR = self.app.ressources # alias
       CP = self.app.parametres # alias
       # ******************* réglages *****************
       self.boutonInitialiser = Button (panneau, bd=4, 
                        command=self.initialiser, font = CR.fonteUsuel, 
                        text = "(ré)INITIALISER", pady=10 )
       self.boutonInitialiser.grid(column=1, row=2, rowspan=2, padx=5)

       self.barreVitesse = Scale(panneau, orient=HORIZONTAL, from_=0,
                    to=100, label="durée d'un pas (u = 0,01 s)", width=15,
                    length =280, font= CR.fonteScale, tickinterval=20,
                    resolution=5, command=self.initBarres, relief=SOLID)
       self.barreVitesse.set(10)
       self.barreVitesse.grid(column=0, row=2, pady=3, padx= 5, sticky=W)

       self.barreDelta = Scale(panneau, orient=HORIZONTAL, from_=40,
                     to=280, label="nombre de pas du dessin", width=15,
                     length =280, font= CR.fonteScale, tickinterval=40,
                     resolution=10, command=self.initBarres, relief=SOLID)
       self.barreDelta.set(50)
       self.barreDelta.grid(column=0, row=3, pady=3, padx= 5, sticky=W)

       # ******************* contrôles *****************
       self.labelP = ["","","",""]
       self.labelP[0] = Label (panneau,
                      text=CR.message[0],
                      bg = CR.couleurPanneau,  pady=3, font = CR.fonteUsuel,
                      anchor=W, state=DISABLED )
       self.labelP[0].grid (column=0, row=4, padx = 5, columnspan=2, 
                            sticky=W )
                            
       self.labelP[1] = Label (panneau,
                      text=CR.message[1],
                      bg = CR.couleurPanneau, pady=3, font = CR.fonteUsuel,
                      anchor=W, state=DISABLED )
       self.labelP[1].grid (column=0, row=5, padx = 5, columnspan=2, 
                            sticky=W )
       
       self.labelP[2] = Label (panneau,
                      text=CR.message[2],
                      bg = CR.couleurPanneau, pady=3, font = CR.fonteUsuel,
                      anchor=W, state=DISABLED )
       self.labelP[2].grid (column=0, row=6, padx = 5, columnspan=2, 
                            sticky=W )

       self.labelP[3] = Label (panneau,
                      text=CR.message[3],
                      bg = CR.couleurPanneau, pady=3, font = CR.fonteUsuel,
                      anchor=W, state=DISABLED )
       self.labelP[3].grid (column=0, row=7, padx = 5,columnspan=2, 
                      sticky=W )

       # ************* stop and go **********************
       frameCadreActuel = Frame (panneau, bd=1, relief=SOLID)
       frameCadreActuel.grid (column=0, row=8, columnspan=2, pady=5)

       cadreActuel = LabelFrame (frameCadreActuel, text=" Point courant ",
                      font=CR.fonteUsuel)
       cadreActuel.grid (column=0, row=0, padx=5, pady=5)
       
       self.labeltActuel = Label (cadreActuel, text="", width =10,
                       anchor=W, pady=5, font = CR.fonteUsuel)
       self.labeltActuel.grid (column=1, row=0, sticky=W )
       
       self.boutonStopGo = Button (cadreActuel, bd=4, command=self.stopGo,
                font = CR.fonteUsuel, text = "  Stop & Go  ", state=DISABLED)
       CR.bgStopGo = self.boutonStopGo.cget("bg")
       self.boutonStopGo.grid(column=1, row=1, pady=10, padx=10)

       self.labelxActuel = Label (cadreActuel, text="", width =10,
                        anchor=W, pady=5, font = CR.fonteUsuel)
       self.labelxActuel.grid (column=0, row=0, pady=3, sticky=W, padx=40)

       self.labelyActuel = Label (cadreActuel, text="", width =10,
                        anchor=W, pady=5, font = CR.fonteUsuel)
       self.labelyActuel.grid (column=0, row=1, pady=3, sticky=W, padx=40)


    def initialiser (self) :
        CR = self.app.ressources # alias
        CP = self.app.parametres # alias
        CP.flagStopGo = False
        self.barreDelta.config (state=NORMAL)
        self.barreVitesse.config (state=NORMAL)
        self.labelP[0].config(text=CR.message[0], state=DISABLED )
        self.labelP[1].config(text=CR.message[1], state=DISABLED )
        self.labelP[2].config(text=CR.message[2], state=DISABLED )
        self.labelP[3].config(text=CR.message[3], state=DISABLED )
        self.app.canevas.delete("courbe")
        self.labelxActuel.config(text="")
        self.labelyActuel.config(text="")
        self.labeltActuel.config(text="")
        self.boutonStopGo.config(state=DISABLED)
        self.boutonStopGo.config(
                           bg=CR.bgStopGo, activebackground=CR.bgStopGo)
        self.app.canevas.tag_raise("présentation","carré")                   
        CP.etat = -1

    def initBarres (self, Event=None) :
        CP = self.app.parametres # alias
        CP.setVitesse (self.barreVitesse.get())
        CP.setPas (self.barreDelta.get())

    def normalDisabled (self, fin=True):
        CP = self.app.parametres # alias
        if (fin) :
           self.barreDelta.config (state=DISABLED)
           self.barreVitesse.config (state=DISABLED)
        else : # changer les paramètres sur les mêmes points
           self.barreDelta.config (state=NORMAL)
           self.barreVitesse.config (state=NORMAL)
        if (CP.flagStopGo) :
            self.app.boutonQuitter.config (state=DISABLED)
            self.boutonInitialiser.config (state=DISABLED)
        else:
            self.app.boutonQuitter.config (state=NORMAL)
            self.boutonInitialiser.config (state=NORMAL)

    def stopGo (self):
        CR = self.app.ressources # alias
        CP = self.app.parametres # alias
        # le flag est posé si le thread est actif
        if (CP.flagStopGo) : # le processus est actif
            CR.verrou.acquire()  # protection en écriture
            CP.flagStopGo = False
            CR.verrou.release()
            self.setBgBoutonStopGo (CR.bgStopGoArret)
            self.normalDisabled ()

        else : # créer un processus de suite
            CP.flagStopGo = True
            self.setBgBoutonStopGo (CR.bgStopGoTrace)

            self.normalDisabled()
            t=Thread(target = self.app.canevas.stopGo)
            t.start()

    def setLabelPnormal (self, n) :
        self.labelP[n].config ( state = NORMAL )

    def setLabelPxy (self, n) :
        CR = self.app.ressources # alias
        CP = self.app.parametres # alias
        coords = CR.coord2str (CP.Point[n][XC], CP.Point[n][YC])
        self.labelP[n].config(text=CR.message[n]+
                             " : x = "+coords[XC]+"  y = "+coords[YC])
    def setLabelActuel (self, x, y) :
        CR = self.app.ressources # alias
        CP = self.app.parametres # alias
        coords = CR.coord2str (x, y)
        self.labeltActuel.config (text= "t = "+str(CP.t)+" / "+str(CP.pas))
        self.labelxActuel.config (text= "x = "+coords[XC])
        self.labelyActuel.config (text= "y = "+coords[YC])

    def setBgBoutonStopGo (self, couleur) :
        self.boutonStopGo.config(bg= couleur, activebackground=couleur)

# ********************************************************
# ************ l'Application et sa Fenêtre ***************
# ********************************************************
class Application (Tk) :
    def __init__(self) :
        Tk.__init__(self) # hérite de Tk --> appeler le __init__
        self.protocol ("WM_DELETE_WINDOW",  self.quitter) # iconisation
        self.title("courbes de Bézier cubiques")
        self.resizable(False, False)

        # ********* ressources / paramètres ****************        
        self.ressources = ClasseRessources (self.winfo_screenheight())
        self.parametres = ClasseParametres ()
        CR = self.ressources
        CP = self.parametres
        # ***************** panneaux ***********************
        panneauGauche = Frame (self, bd=4, relief="groove",
                        bg = CR.couleurPanneau)
        panneauGauche.grid (column=0, row=0 )
        panneauDroite = Frame (self, bd=4, relief="groove",
                        bg = CR.couleurPanneau)
        panneauDroite.grid (column=1, row=0, sticky = N  )

        # ****************** titre ********************
        labelTitre = Label (panneauDroite,
           text="Courbes de Bézier \n Cubiques", bg = CR.couleurPanneau, 
                               padx = 3, pady = 5, font = CR.fonteTitre)
        labelTitre.grid (column=0, row=0, columnspan=2, sticky = N)

        # ******************* contrôles *****************
        self.controles = ClasseControles (panneauDroite, self)

        # ************** canevas **********************
        self.canevas = ClasseCanevas(panneauGauche, self)
        self.canevas.config (height = CR.hauteur, width=CR.hauteur,
                                         bg=CR.couleurCanevas, bd=0)
        self.canevas.grid (column=0, row=0)

        # **************** quitter **********************

        self.boutonQuitter = Button (panneauDroite, bd=4, fg = "#800000",
             command=self.quitter,font = CR.fonteUsuel, text = "QUITTER")
        self.boutonQuitter.grid(column=0, row=9, columnspan=2, pady=10)

        # ************** réglages de mise en page *******************
        
        self.miseEnPage(panneauGauche, panneauDroite)
        # derniers réglages        
        self.controles.initialiser()
        self.controles.normalDisabled()
        CP.etat = -2

    def miseEnPage (self, pG, pD) :
        self.update_idletasks() # nécessaire pour calculer les dimensions
        # remplissage du panneau de droite
        py1 = pD.winfo_reqheight()
        py2 =  pG.winfo_reqheight()
        labelAjuste = Label (pD, bitmap="@barre400.xbm",
                    height=py2-py1, bg=self.ressources.couleurPanneau)
        labelAjuste.grid (column=0, row=1, columnspan=2)
        # mettre l'affichage en milieu d'écran
        h =  self.winfo_reqheight()
        w =  self.winfo_reqwidth()
        dsh = self.winfo_screenheight() - h
        dsw = self.winfo_screenwidth() - w
        geometrie = str(w)+"x"+str(h)+"+"+str(dsw//2)+"+"+str(dsh//2)        
        self.geometry(geometrie)

    def quitter(self) :
        self.quit()
# **************** fin de Application *******************

# ***********partie maître de l'application ***************
bezierCubique = Application()
bezierCubique.mainloop()
bezierCubique.destroy()
# *********************************************************

